package facturation.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import facturation.modele.Article;

public class ArticleDAO {
	/**
	 * Connexion à la base de donnée.
	 */
	private ConnectionFactory connectionFactory;

	public ArticleDAO(ConnectionFactory connectionFactory) {
		this.connectionFactory= connectionFactory;
	}

	/**
	 * Insère un élément dans la base.
	 * @param article
	 * @throws SQLException
	 */
	public void create(Article article) throws SQLException {
		Connection connection = connectionFactory.open();
		PreparedStatement pst = null;
		try {
			pst = connection
					.prepareStatement("insert into Article values(?,?,?)");
			pst.setInt(1, article.getIdArticle());
			pst.setString(2, article.getDesignation());
			pst.setDouble(3, article.getPrixTTC());
			pst.executeUpdate();
		} finally {
			if (pst != null)
				pst.close();
			connection.close();
		}
	}

	/**
	 * Met à jour un article déjà existant.
	 * @param article
	 * @throws SQLException
	 */
	public void updateArticle(Article article) throws SQLException {
		Connection connection = connectionFactory.open();
		PreparedStatement pst = null;
		try {
			pst = connection
					.prepareStatement("update Article set designation=?, prixttc= ? where idArticle= ?");
			pst.setString(1, article.getDesignation());
			pst.setDouble(2, article.getPrixTTC());
			pst.setInt(3, article.getIdArticle());
			pst.executeUpdate();
		} finally {
			if (pst != null)
				pst.close();
			connection.close();
		}
	}
	
	/**
	 * Renvoie l'article qui a un numéro donné.
	 * @param id
	 * @return
	 * @throws SQLException
	 */
	public Article findArticlebyId(int id) throws SQLException {
		Connection connection = connectionFactory.open();
		PreparedStatement pst = null;
		ResultSet r= null;
		Article result= null;
		try {
			pst = connection
					.prepareStatement("select * from  Article where idArticle= ?");
			
			pst.setInt(1, id);
			r= pst.executeQuery();
			// 1 résultat max.
			if (r.next()) {
				result= new Article();
				result.setIdArticle(r.getInt("idArticle"));
				result.setDesignation(r.getString("designation"));
				result.setPrixTTC(r.getDouble("prixttc"));
			}	
		} finally {
			if (r!= null)
				r.close();
			if (pst != null)
				pst.close();
			connection.close();
		}
		return result;
	}
	
	/**
	 * Renvoie le ou les articles qui ont une désignation donnée.
	 * @param designation
	 * @return
	 * @throws SQLException
	 */
	public List<Article> findArticlebyDesignation(String designation) throws SQLException {
		Connection connection = connectionFactory.open();
		PreparedStatement pst = null;
		ResultSet r= null;
		List<Article> result= new ArrayList<Article>();
		try {
			pst = connection
					.prepareStatement("select * from  Article where designation= ?");
			
			pst.setString(1, designation);
			r= pst.executeQuery();
			while (r.next()) {
				Article a= new Article();
				a.setIdArticle(r.getInt("idArticle"));
				a.setDesignation(r.getString("designation"));
				a.setPrixTTC(r.getDouble("prixttc"));
				result.add(a);
			}	
		} finally {
			if (r!= null)
				r.close();
			if (pst != null)
				pst.close();
			connection.close();
		}
		return result;
	}
	
	public List<Article> findAllArticles() throws SQLException {
		Connection connection = connectionFactory.open();
		PreparedStatement pst = null;
		ResultSet r= null;
		List<Article> result= new ArrayList<Article>();
		try {
			pst = connection
					.prepareStatement("select * from  Article");
			r= pst.executeQuery();
			while (r.next()) {
				Article a= new Article();
				a.setIdArticle(r.getInt("idArticle"));
				a.setDesignation(r.getString("designation"));
				a.setPrixTTC(r.getDouble("prixttc"));
				result.add(a);
			}	
		} finally {
			if (r!= null)
				r.close();
			if (pst != null)
				pst.close();
			connection.close();
		}
		return result;
	}
	
	/**
	 * Détruit l'article d'identifiant "id".
	 * @param id
	 * @throws SQLException
	 */
	public void delete(int id) throws SQLException {
		Connection connection = connectionFactory.open();
		PreparedStatement pst = null;
		try {
			pst = connection
					.prepareStatement("delete from Article where idArticle= ?");
			pst.setInt(1, id);
			pst.executeUpdate();
		} finally {
			if (pst != null)
				pst.close();
			connection.close();
		}
	}
	
	
	public int getNextId() throws SQLException {
		String idName= "idArticle";
		String tableName= "Article";
		
		return new IDGeneratorDAO(connectionFactory).getNextId(idName, tableName);
		
	}

	
}
