package facturation.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;

import facturation.modele.Article;
import facturation.modele.Client;

public class ClientDAO {
	private ConnectionFactory connectionFactory;

	public ClientDAO(ConnectionFactory connectionFactory) {
		this.connectionFactory= connectionFactory;
	}

	public int getNextId() throws SQLException {
		return new IDGeneratorDAO(connectionFactory).getNextId("idClient", "client");
	}

	public void create(Client client) throws SQLException {
		Connection connection = connectionFactory.open();
		PreparedStatement pst = null;
		try {
			pst = connection
					.prepareStatement("insert into Client values(?,?,?,?)");
			pst.setInt(1, client.getIdClient());
			pst.setString(2, client.getNom());
			pst.setString(3, client.getAdresse());
			pst.setInt(4, client.getPointsDeFidelite());

			pst.executeUpdate();
		} finally {
			if (pst != null)
				pst.close();
			connection.close();
		}
		
	}

	public Client findClientById(int id) throws SQLException {
		Connection connection = connectionFactory.open();
		PreparedStatement pst = null;
		ResultSet r= null;
		Client result= null;
		try {
			pst = connection
					.prepareStatement("select * from  Client where idClient= ?");
			
			pst.setInt(1, id);
			r= pst.executeQuery();
			// 1 résultat max.
			if (r.next()) {
				result = extractClientFromResultSet(r);
			}	
		} finally {
			if (r!= null)
				r.close();
			if (pst != null)
				pst.close();
			connection.close();
		}
		return result;
	}

	/**
	 * Récupère le client sur la ligne courante.
	 * @param r
	 * @return
	 * @throws SQLException
	 */
	private Client extractClientFromResultSet(ResultSet r) throws SQLException {
		Client result;
		result= new Client();
		result.setIdClient(r.getInt("idClient"));
		result.setNom(r.getString("nom"));
		result.setAdresse(r.getString("adresse"));
		result.setPointsDeFidelite(r.getInt("pointsDeFidelite"));
		return result;
	}

	public List<Client> findClientByNom(String nom) throws SQLException {
		Connection connection = connectionFactory.open();
		PreparedStatement pst = null;
		ResultSet r= null;
		ArrayList<Client> result= new ArrayList<Client>();
		
		try {
			// On cherche les clients dont les noms commencent d'une certaine manière. 
			// On peut utiliser l'opérateur SQL like...
			pst = connection
					.prepareStatement("select * from  Client where nom like ?");
			
			pst.setString(1, nom+"%");
			r= pst.executeQuery();
			
			while (r.next()) {
				result.add(extractClientFromResultSet(r));
			}	
		} finally {
			if (r!= null)
				r.close();
			if (pst != null)
				pst.close();
			connection.close();
		}
		return result;
	}

	/**
	 * Met à jour le client courant.
	 * @param clientCourant
	 * @throws SQLException 
	 */
	public void update(Client client) throws SQLException {
		Connection connection = connectionFactory.open();
		PreparedStatement pst = null;
		try {
			pst = connection
					.prepareStatement("update client set nom= ?, adresse= ?, pointsDeFidelite= ? where idClient= ?");
			pst.setString(1, client.getNom());
			pst.setString(2, client.getAdresse());
			pst.setInt(3, client.getPointsDeFidelite());
			pst.setInt(4, client.getIdClient());
			
			pst.executeUpdate();
		} finally {
			if (pst != null)
				pst.close();
			connection.close();
		}		
	}
}
