package facturation.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;

import facturation.modele.Facture;

public class FactureDAO {
	private ConnectionFactory connectionFactory;

	
	public FactureDAO(ConnectionFactory connectionFactory) {
		this.connectionFactory = connectionFactory;
	}

	/**
	 * Sauvegarde d'une facture.
	 * @param facture
	 * @throws SQLException 
	 */
	public void save(Facture facture) throws SQLException {
		Connection connection = connectionFactory.open();
		PreparedStatement factureStatement = null;
		PreparedStatement ligneFactureStatement= null;
		try {
			// Normalement, nous devrions encadrer toutes les méthodes de la
			// logique par des transactions.
			// Nous verrons bientôt comment le faire proprement.
			// On passe en mode transactionnel.
			connection.setAutoCommit(false);

			// D'abord, sauver la facture elle-même.
			factureStatement = connection
					.prepareStatement("insert into Facture values (?,?,?)");
			factureStatement.setInt(1, facture.getIdFacture());
			factureStatement.setInt(2, facture.getClient().getIdClient());
			factureStatement.setDouble(3, facture.getPrixTotal());
			factureStatement.executeUpdate();
			// Ouf !
			// Sauvegarde des lignes de la facture !
			// Dans notre base, la clef primaire des lignes est un couple : (idFacture, idLigne)
			// (i.e. nous représentons un tableau de lignes).
			ligneFactureStatement= connection.prepareStatement("insert into ligneFacture (idFacture, idLigne,  idProduit) values (?,?,?)");
			// Le numéro de la facture est fixe !
			ligneFactureStatement.setInt(1, facture.getIdFacture());
			
			for (int i=0; i < facture.getArticles().size(); i++) {
				ligneFactureStatement.setInt(2, i);
				ligneFactureStatement.setDouble(3, facture.getArticles().get(i).getIdArticle());
				ligneFactureStatement.executeUpdate();
			}
			
			// On termine la transaction.
			connection.commit();
		} catch (SQLException e) {
			// En cas de problème, on annule tout !
			if (connection != null) {
				connection.rollback();
			}
			// On laisse passer l'exception quand même :
			throw e;
		} finally {
			if (ligneFactureStatement != null)
				ligneFactureStatement.close();
			if (factureStatement != null)
				factureStatement.close();
			connection.close();
		}

	}

	public int getNextId() throws SQLException {
		return new IDGeneratorDAO(connectionFactory).getNextId("idFacture",
				"facture");
	}

}
