package facturation.ui;

import java.util.List;

import facturation.logique.FacturationLogique;
import facturation.modele.Article;
import facturation.modele.Client;
import simpleIO.SimpleIO;

/**
 * Interface texte de l'application de facturation.
 * @author rosmord
 *
 */
public class FacturationTextUI {
	/**
	 * nos entrées/sorties.
	 */
	private SimpleIO simpleIO;
	private FacturationLogique facturationLogique;
	
	public FacturationTextUI(FacturationLogique logique) {
		simpleIO= new SimpleIO();
		this.facturationLogique= logique;
	}

	public void menuPrincipal() {
		String reponse= "c";
		
		while (!  "q".equals(reponse)) {
			simpleIO.ecrire("Menu\n");
			simpleIO.ecrire("\tc- créer (C)lient");
			simpleIO.ecrire("\tf- créer (F)acture");
			simpleIO.ecrire("\tq- (Q)uitter");
			reponse= simpleIO.lireLigne("votre choix :");
			if (reponse.startsWith("c"))
				creerClient();
			else if (reponse.startsWith("f")) {
				creerFacture();
			}
		}
	}
	
	

	public void creerClient() {
		facturationLogique.reinitialiserClient();
		// Demander nom
		String nom= simpleIO.lireLigne("nom :");
		facturationLogique.fixerNomClient(nom);
		// Demander adresse
		String adresse= simpleIO.lireLigne("adresse :");
		facturationLogique.fixerAdresseClient(adresse);
		// valider
		facturationLogique.creerClient();
		// Montrer le résultat.
		simpleIO.ecrire("Client créé avec l'identifiant "+ facturationLogique.getClientCourant().getIdClient());
	}
	
	public void chercherClient() {
		// Demander nom
		String nom= simpleIO.lireLigne("nom du client à chercher :");
		List<Client> clients= facturationLogique.chercherClientParNom(nom);
		// Montrer client(s)
		for (Client c: clients) {
			simpleIO.ecrire(c.toString());
			simpleIO.ecrire("\n");
		}
		// Demander id
		int id= simpleIO.lireEntier("id du client");
		facturationLogique.selectionnerClient(id);

		// Remarque un peu acrobatique : pour permettre facilement d'annuler 
		// l'opération en cours, on peut envisager d'utiliser les exceptions.
		// l'avantage est que, par exemple, on peut annuler creerFacture (ou autre)
		// depuis creerClient, sans que ces méthodes doivent traiter l'annulation.
		// Il suffit ensuite de gerer les "AnnulationException" dans la boucle principale.
	}
	
	public void creerFacture() {
		String choix= "x";
		facturationLogique.initialiserFacture();
		// Selectionner un client
		chercherClient();
		// Tant que nécessaire.
		while (! "c".equals(choix) && ! "v".equals(choix)) {
			// Montrer catalogue
			List<Article> catalogue= facturationLogique.getCatalogue();
			for (Article a: catalogue) {
				simpleIO.ecrire(a.toString());
				simpleIO.ecrire("\n");
			}
			// Demande ID produit ou choix de menu.
			choix= simpleIO.lireLigne("Entrer 'v' pour valider, 'c' pour annuler, ou l'id du produit pour l'ajouter");
			if (! "v".equals(choix) && ! "c".equals(choix)) {
				int id= Integer.parseInt(choix);
				// On a l'id. Ajouter le produit.
				facturationLogique.ajouterProduit(id);
			}
		}
		if ("v".equals(choix)) {
			// Valider facture
			facturationLogique.validerFacture();
			// Afficher prix total
			System.out.println(facturationLogique.getTotalFacture());
		} else {
			simpleIO.ecrire("commande annulée");
		}
	}
}
